# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacOauth1ConsumerTest(rbac_base.IdentityV3RbacBaseTests,
                                       metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacOauth1ConsumerTest, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.oauth_consumers_client
        cls.admin_client = cls.os_system_admin.oauth_consumers_client

    def consumer(self):
        return {"description": data_utils.arbitrary_string()}

    @abc.abstractmethod
    def test_identity_create_consumer(self):
        """Test identity:create_consumer policy.

        This test must check:
          * whether the persona can create a consumer
        """
        pass

    @abc.abstractmethod
    def test_identity_get_consumer(self):
        """Test identity:get_consumer policy.

        This test must check:
          * whether the persona can get a consumer
        """
        pass

    @abc.abstractmethod
    def test_identity_list_consumers(self):
        """Test identity:list_consumers policy.

        This test must check:
          * whether the persona can list all consumers
        """
        pass

    @abc.abstractmethod
    def test_identity_update_consumer(self):
        """Test identity:update_consumer policy.

        This test must check:
          * whether the persona can update a
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_consumer(self):
        """Test identity:delete_consumer policy.

        This test must check
          * whether the persona can delete a consumer
        """
        pass


class SystemAdminTests(
    IdentityV3RbacOauth1ConsumerTest, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_consumer(self):
        resp = self.do_request('create_consumer',
                               expected_status=201,
                               **self.consumer())
        self.addCleanup(self.client.delete_consumer,
                        resp['consumer']['id'])

    def test_identity_get_consumer(self):
        consumer = self.admin_client.create_consumer(
            **self.consumer())['consumer']
        self.addCleanup(self.admin_client.delete_consumer, consumer['id'])
        resp = self.do_request('show_consumer', consumer_id=consumer['id'])
        self.assertEqual(resp['consumer']['id'], consumer['id'])

    def test_identity_list_consumers(self):
        consumer = self.admin_client.create_consumer(
            **self.consumer())['consumer']
        self.addCleanup(self.admin_client.delete_consumer, consumer['id'])
        resp = self.do_request('list_consumers')
        self.assertIn(consumer['id'], set(c['id'] for c in resp['consumers']))

    def test_identity_update_consumer(self):
        consumer = self.client.create_consumer(**self.consumer())['consumer']
        self.addCleanup(self.client.delete_consumer, consumer['id'])
        self.do_request('update_consumer',
                        consumer_id=consumer['id'],
                        description=data_utils.arbitrary_string())

    def test_identity_delete_consumer(self):
        consumer = self.client.create_consumer(**self.consumer())['consumer']
        self.do_request('delete_consumer',
                        expected_status=204,
                        consumer_id=consumer['id'])


class SystemMemberTests(SystemAdminTests):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_consumer(self):
        self.do_request('create_consumer',
                        expected_status=exceptions.Forbidden,
                        **self.consumer())

    def test_identity_update_consumer(self):
        consumer = self.admin_client.create_consumer(
            **self.consumer())['consumer']
        self.addCleanup(self.admin_client.delete_consumer, consumer['id'])
        self.do_request('update_consumer',
                        expected_status=exceptions.Forbidden,
                        consumer_id=consumer['id'],
                        description=data_utils.arbitrary_string())

    def test_identity_delete_consumer(self):
        consumer = self.admin_client.create_consumer(
            **self.consumer())['consumer']
        self.do_request('delete_consumer',
                        expected_status=exceptions.Forbidden,
                        consumer_id=consumer['id'])


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemMemberTests):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_consumer(self):
        consumer = self.admin_client.create_consumer(
            **self.consumer())['consumer']
        self.addCleanup(self.admin_client.delete_consumer, consumer['id'])
        self.do_request('show_consumer',
                        expected_status=exceptions.Forbidden,
                        consumer_id=consumer['id'])

    def test_identity_list_consumers(self):
        consumer = self.admin_client.create_consumer(
            **self.consumer())['consumer']
        self.addCleanup(self.admin_client.delete_consumer, consumer['id'])
        self.do_request('list_consumers',
                        expected_status=exceptions.Forbidden)


class DomainManagerTests(DomainAdminTests):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import requests


class KeycloakClient(object):
    def __init__(self, keycloak_url, keycloak_username, keycloak_password,
                 realm='master', ca_certs_file=False):
        self.keycloak_url = keycloak_url
        self.keycloak_username = keycloak_username
        self.keycloak_password = keycloak_password
        self.session = requests.session()
        self.realm = realm
        self.ca_certs_file = ca_certs_file
        self._admin_auth()

    @property
    def url_base(self):
        return self.keycloak_url + '/admin/realms'

    @property
    def token_endpoint(self):
        return self.keycloak_url + \
            f'/realms/{self.realm}/protocol/openid-connect/token'

    @property
    def discovery_endpoint(self):
        return self.keycloak_url + \
            f'/realms/{self.realm}/.well-known/openid-configuration'

    def _construct_url(self, path):
        return self.url_base + f'/{self.realm}/{path}'

    def _admin_auth(self):
        params = {
            'grant_type': 'password',
            'client_id': 'admin-cli',
            'username': self.keycloak_username,
            'password': self.keycloak_password,
            'scope': 'openid',
        }
        r = requests.post(
            self.token_endpoint,
            data=params,
            verify=self.ca_certs_file).json()

        headers = {
            'Authorization': ("Bearer %s" % r['access_token']),
            'Content-Type': 'application/json'
        }
        self.session.headers.update(headers)
        return r

    def create_user(self, email, first_name, last_name):
        self._admin_auth()
        data = {
            'username': email,
            'email': email,
            'firstName': first_name,
            'lastName': last_name,
            'enabled': True,
            'emailVerified': True,
            'credentials': [{
                'value': 'secret',
                'type': 'password',
            }],
            'requiredActions': []
        }
        return self.session.post(
            self._construct_url('users'),
            json=data, verify=self.ca_certs_file)

    def delete_user(self, username):
        self._admin_auth()
        data = {
            'id': username,
        }
        return self.session.delete(
            self._construct_url('users'),
            json=data, verify=self.ca_certs_file)



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacPolicyTests(rbac_base.IdentityV3RbacBaseTests,
                                metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacPolicyTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.policies_client
        admin_client = cls.os_system_admin
        cls.admin_policies_client = admin_client.policies_client

    def policy(self):
        return {
            'blob': data_utils.rand_uuid_hex(),
            'type': data_utils.rand_uuid_hex()
        }

    @abc.abstractmethod
    def test_identity_create_policy(self):
        """Test identity:create_policy policy.

        This test must check:
          * whether the persona can create a policy
        """
        pass

    @abc.abstractmethod
    def test_identity_get_policy(self):
        """Test identity:get_policy policy.

        This test must check:
          * whether the persona can get a policy
          * whether the persona can get a policy that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_policies(self):
        """Test identity:list_policies policy.

        This test must check:
          * whether the persona can list all policies
        """
        pass

    @abc.abstractmethod
    def test_identity_update_policy(self):
        """Test identity:update_policy policy.

        This test must check:
          * whether the persona can update a policy
          * whether the persona can update a policy that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_policy(self):
        """Test identity:delete_policy policy.

        This test must check
          * whether the persona can delete a policy
          * whether the persona can delete a policy that does not exist
        """
        pass


class SystemAdminTests(IdentityV3RbacPolicyTests, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_policy(self):
        policy_id = self.do_request(
            'create_policy', expected_status=201,
            **self.policy())['policy']['id']
        self.addCleanup(
            self.admin_policies_client.delete_policy,
            policy_id=policy_id)

    def test_identity_get_policy(self):
        policy_id = self.admin_policies_client.create_policy(
            **self.policy())['policy']['id']
        self.addCleanup(
            self.admin_policies_client.delete_policy,
            policy_id=policy_id)
        self.do_request('show_policy', policy_id=policy_id)
        # user gets a 404 for nonexistent policy
        self.do_request('show_policy', expected_status=exceptions.NotFound,
                        policy_id=data_utils.rand_uuid_hex())

    def test_identity_list_policies(self):
        policy_id = self.admin_policies_client.create_policy(
            **self.policy())['policy']['id']
        self.addCleanup(
            self.admin_policies_client.delete_policy,
            policy_id=policy_id)
        resp = self.do_request('list_policies')
        self.assertIn(policy_id, [e['id'] for e in resp['policies']])

    def test_identity_update_policy(self):
        policy_id = self.admin_policies_client.create_policy(
            **self.policy())['policy']['id']
        self.addCleanup(
            self.admin_policies_client.delete_policy,
            policy_id=policy_id)
        self.do_request('update_policy',
                        policy_id=policy_id,
                        blob=data_utils.rand_uuid_hex())
        # user gets a 404 for nonexistent policy
        self.do_request('update_policy', expected_status=exceptions.NotFound,
                        policy_id=data_utils.rand_uuid_hex(),
                        blob=data_utils.rand_uuid_hex())

    def test_identity_delete_policy(self):
        policy_id = self.admin_policies_client.create_policy(
            **self.policy())['policy']['id']
        self.do_request('delete_policy', expected_status=204,
                        policy_id=policy_id)
        # user gets a 404 for nonexistent policy
        self.do_request('delete_policy', expected_status=exceptions.NotFound,
                        policy_id=data_utils.rand_uuid_hex())


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_policy(self):
        self.do_request(
            'create_policy', expected_status=exceptions.Forbidden,
            **self.policy())

    def test_identity_update_policy(self):
        policy_id = self.admin_policies_client.create_policy(
            **self.policy())['policy']['id']
        self.addCleanup(
            self.admin_policies_client.delete_policy,
            policy_id=policy_id)
        self.do_request('update_policy', expected_status=exceptions.Forbidden,
                        policy_id=policy_id,
                        blob=data_utils.rand_uuid_hex())
        # user gets a 403 for nonexistent policy
        self.do_request('update_policy', expected_status=exceptions.Forbidden,
                        policy_id=data_utils.rand_uuid_hex(),
                        blob=data_utils.rand_uuid_hex())

    def test_identity_delete_policy(self):
        policy_id = self.admin_policies_client.create_policy(
            **self.policy())['policy']['id']
        self.do_request('delete_policy',
                        expected_status=exceptions.Forbidden,
                        policy_id=policy_id)
        # user gets a 403 for nonexistent policy
        self.do_request('delete_policy', expected_status=exceptions.Forbidden,
                        policy_id=data_utils.rand_uuid_hex())


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_policy(self):
        policy_id = self.admin_policies_client.create_policy(
            **self.policy())['policy']['id']
        self.addCleanup(
            self.admin_policies_client.delete_policy,
            policy_id=policy_id)
        self.do_request('show_policy', expected_status=exceptions.Forbidden,
                        policy_id=policy_id)
        # user gets a 403 for nonexistent policy
        self.do_request('show_policy', expected_status=exceptions.Forbidden,
                        policy_id=data_utils.rand_uuid_hex())

    def test_identity_list_policies(self):
        policy_id = self.admin_policies_client.create_policy(
            **self.policy())['policy']['id']
        self.addCleanup(
            self.admin_policies_client.delete_policy,
            policy_id=policy_id)
        self.do_request('list_policies', expected_status=exceptions.Forbidden)


class DomainManagerTests(DomainAdminTests, base.BaseIdentityTest):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import http.client
import json
from lxml import etree
from tempest import config
from tempest.lib.common.utils import data_utils
import testtools
import urllib

from keystone_tempest_plugin.tests import base


CONF = config.CONF


class TestSaml2EcpFederatedAuthentication(base.BaseIdentityTest):

    ECP_SAML2_NAMESPACES = {
        'ecp': 'urn:oasis:names:tc:SAML:2.0:profiles:SSO:ecp',
        'S': 'http://schemas.xmlsoap.org/soap/envelope/',
        'paos': 'urn:liberty:paos:2003-08'
    }

    ECP_SERVICE_PROVIDER_CONSUMER_URL = ('/S:Envelope/S:Header/paos:Request/'
                                         '@responseConsumerURL')

    ECP_IDP_CONSUMER_URL = ('/S:Envelope/S:Header/ecp:Response/'
                            '@AssertionConsumerServiceURL')

    ECP_RELAY_STATE = '//ecp:RelayState'

    def _setup_settings(self):
        self.idp_id = CONF.fed_scenario.idp_id
        self.idp_remote_ids = CONF.fed_scenario.idp_remote_ids
        self.idp_url = CONF.fed_scenario.idp_ecp_url
        self.keystone_v3_endpoint = CONF.identity.uri_v3
        self.password = CONF.fed_scenario.idp_password
        self.protocol_id = CONF.fed_scenario.protocol_id
        self.username = CONF.fed_scenario.idp_username

        self.mapping_remote_type = CONF.fed_scenario.mapping_remote_type
        self.mapping_user_name = CONF.fed_scenario.mapping_user_name
        self.mapping_group_name = CONF.fed_scenario.mapping_group_name
        self.mapping_group_domain_name = \
            CONF.fed_scenario.mapping_group_domain_name

        # NOTE(knikolla): Authentication endpoint for keystone. If not set,
        # will be autodetected.
        self.auth_url = None

    def _setup_idp(self):
        idp = self.idps_client.create_identity_provider(
            self.idp_id, remote_ids=self.idp_remote_ids, enabled=True)
        self.addCleanup(
            self.keystone_manager.domains_client.delete_domain,
            idp['identity_provider']['domain_id'])
        self.addCleanup(
            self.keystone_manager.domains_client.update_domain,
            idp['identity_provider']['domain_id'], enabled=False)
        self.addCleanup(
            self.idps_client.delete_identity_provider, self.idp_id)

    def _setup_mapping(self):
        self.mapping_id = data_utils.rand_uuid_hex()
        rules = [{
            'local': [
                {
                    'user': {'name': self.mapping_user_name}
                },
                {
                    'group': {
                        'domain': {'name': self.mapping_group_domain_name},
                        'name': self.mapping_group_name
                    }
                }
            ],
            'remote': [
                {
                    'type': self.mapping_remote_type
                }
            ]
        }]
        mapping_ref = {'rules': rules}
        self.mappings_client.create_mapping_rule(self.mapping_id, mapping_ref)
        self.addCleanup(
            self.mappings_client.delete_mapping_rule, self.mapping_id)

    def _setup_protocol(self):
        self.idps_client.add_protocol_and_mapping(
            self.idp_id, self.protocol_id, self.mapping_id)
        self.addCleanup(
            self.idps_client.delete_protocol_and_mapping,
            self.idp_id,
            self.protocol_id)

    def setUp(self):
        super(TestSaml2EcpFederatedAuthentication, self).setUp()
        self._setup_settings()

        # Reset client's session to avoid getting garbage from another runs
        self.saml2_client.reset_session()

        # Setup identity provider, mapping and protocol
        self._setup_idp()
        self._setup_mapping()
        self._setup_protocol()

    def _str_from_xml(self, xml, path):
        item = xml.xpath(path, namespaces=self.ECP_SAML2_NAMESPACES)
        self.assertEqual(1, len(item))
        return item[0]

    def _get_sp_authn_request(self):
        resp = self.saml2_client.send_service_provider_request(
            self.keystone_v3_endpoint, self.idp_id, self.protocol_id)
        self.assertEqual(http.client.OK, resp.status_code)
        saml2_authn_request = etree.XML(resp.content)

        relay_state = self._str_from_xml(
            saml2_authn_request, self.ECP_RELAY_STATE)
        sp_consumer_url = self._str_from_xml(
            saml2_authn_request, self.ECP_SERVICE_PROVIDER_CONSUMER_URL)

        # Perform the authn request to the identity provider
        resp = self.saml2_client.send_identity_provider_authn_request(
            saml2_authn_request, self.idp_url, self.username, self.password)
        self.assertEqual(http.client.OK, resp.status_code)
        saml2_idp_authn_response = etree.XML(resp.content)

        idp_consumer_url = self._str_from_xml(
            saml2_idp_authn_response, self.ECP_IDP_CONSUMER_URL)

        # Assert that both saml2_authn_request and saml2_idp_authn_response
        # have the same consumer URL.
        self.assertEqual(sp_consumer_url, idp_consumer_url)

        self.saml2_client.prepare_sp_saml2_authn_response(
            saml2_idp_authn_response, relay_state)

        return saml2_idp_authn_response, sp_consumer_url

    def _request_unscoped_token(self):
        assertion, sp_url = self._get_sp_authn_request()

        # Present the identity provider authn response to the service provider
        resp = self.saml2_client.send_service_provider_saml2_authn_response(
            assertion, sp_url)
        # Must receive a redirect from service provider to the URL where the
        # unscoped token can be retrieved.
        self.assertIn(resp.status_code,
                      [http.client.FOUND, http.client.SEE_OTHER])

        # If this is K2K, don't follow HTTP specs - after the HTTP 302/303
        # response don't repeat the call directed to the Location URL. In this
        # case, this is an indication that SAML2 session is now active and
        # protected resource can be accessed.
        # https://opendev.org/openstack/keystoneauth/src/tag/3.17.1/keystoneauth1/identity/v3/k2k.py#L152
        sp_url = self.auth_url or resp.headers['location']

        # We can receive multiple types of errors here, the response depends on
        # the mapping and the username used to authenticate in the Identity
        # Provider and also in the Identity Provider remote ID validation.
        # If everything works well, we receive an unscoped token.
        resp = (
            self.saml2_client.send_service_provider_unscoped_token_request(
                sp_url))
        self.assertEqual(http.client.CREATED, resp.status_code)
        self.assertIn('X-Subject-Token', resp.headers)
        self.assertNotEmpty(resp.json())

        return resp

    def _test_request_unscoped_token(self):
        self._request_unscoped_token()

    def _test_request_scoped_token(self):
        resp = self._request_unscoped_token()
        token_id = resp.headers['X-Subject-Token']

        projects = self.auth_client.get_available_projects_scopes(
            self.keystone_v3_endpoint, token_id)['projects']
        self.assertNotEmpty(projects)

        # Get a scoped token to one of the listed projects
        self.tokens_client.auth(
            project_id=projects[0]['id'], token=token_id)


class TestSaml2FederatedExternalAuthentication(
    TestSaml2EcpFederatedAuthentication):

    @testtools.skipUnless(CONF.identity_feature_enabled.federation,
                          "Federated Identity feature not enabled")
    @testtools.skipUnless(CONF.identity_feature_enabled.external_idp,
                          "External identity provider is not available")
    @testtools.skipUnless(CONF.fed_scenario.protocol_id == 'mapped',
                          "Protocol not mapped")
    def test_request_unscoped_token(self):
        self._test_request_unscoped_token()

    @testtools.skipUnless(CONF.identity_feature_enabled.federation,
                          "Federated Identity feature not enabled")
    @testtools.skipUnless(CONF.identity_feature_enabled.external_idp,
                          "External identity provider is not available")
    @testtools.skipUnless(CONF.fed_scenario.protocol_id == 'mapped',
                          "Protocol not mapped")
    def test_request_scoped_token(self):
        self._test_request_scoped_token()


class TestK2KFederatedAuthentication(TestSaml2EcpFederatedAuthentication):

    def setUp(self):
        super(TestK2KFederatedAuthentication, self).setUp()
        self._setup_sp()
        self.auth = {'password': data_utils.rand_password()}
        user_id = self.keystone_manager.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            password=self.auth['password'])['user']['id']
        self.addCleanup(
            self.keystone_manager.users_v3_client.delete_user, user_id)
        self.auth['user_id'] = user_id
        idp_info = self.idps_client.show_identity_provider(self.idp_id)
        domain_id = idp_info['identity_provider']['domain_id']
        project_id = self.keystone_manager.identity_providers_client.project_id
        self.auth['project_id'] = project_id
        group = self.keystone_manager.groups_client.create_group(
            name=data_utils.rand_uuid_hex(), domain_id=domain_id)
        self.addCleanup(
            self.keystone_manager.groups_client.delete_group,
            group['group']['id'])
        role = self.keystone_manager.roles_v3_client.create_role(
            name=data_utils.rand_uuid_hex(), project_id=project_id)
        self.addCleanup(
            self.keystone_manager.roles_v3_client.delete_role,
            role['role']['id'])
        self.keystone_manager.roles_v3_client.create_group_role_on_project(
            group_id=group['group']['id'], project_id=project_id,
            role_id=role['role']['id'])
        self.keystone_manager.groups_client.add_group_user(
            group_id=group['group']['id'], user_id=user_id)

    def _setup_settings(self):
        super(TestK2KFederatedAuthentication, self)._setup_settings()
        self.idp_id = 'keystone'
        self.idp_remote_ids = [
            '%s/OS-FEDERATION/saml2/idp' % self.keystone_v3_endpoint]

        self.mapping_remote_type = 'openstack_user'

        self.sp_id = 'keystone'
        self.auth_url = (
            '%s/OS-FEDERATION/identity_providers/%s/protocols/%s/auth'
            ) % (self.keystone_v3_endpoint, self.sp_id, self.protocol_id)
        url = urllib.parse.urlparse(self.keystone_v3_endpoint)
        self.sp_url = '%s://%s/Shibboleth.sso/SAML2/ECP' % (url.scheme,
                                                            url.netloc)

    def _setup_mapping(self):
        if not CONF.fed_scenario.enable_k2k_groups_mapping:
            super(TestK2KFederatedAuthentication, self)._setup_mapping()
            return
        self.mapping_id = data_utils.rand_uuid_hex()
        rules = [{
            'local': [
                {
                    'user': {'name': self.mapping_user_name}
                },
                {
                    'groups': '{1}'
                }
            ],
            'remote': [
                {
                    'type': self.mapping_remote_type
                },
                {
                    "type": 'openstack_groups'
                }
            ]
        }]
        mapping_ref = {'rules': rules}
        self.mappings_client.create_mapping_rule(self.mapping_id, mapping_ref)
        self.addCleanup(
            self.mappings_client.delete_mapping_rule, self.mapping_id)

    def _setup_sp(self):
        self.sps_client.create_service_provider(self.sp_id,
                                                sp_url=self.sp_url,
                                                auth_url=self.auth_url,
                                                enabled=True)
        self.addCleanup(self.sps_client.delete_service_provider, self.sp_id)

    def _get_sp_authn_request(self):
        token = self.tokens_client.get_token(
            user_id=self.auth['user_id'],
            password=self.auth['password'],
            project_id=self.auth['project_id'])
        body = {
            'auth': {
                'identity': {
                    'methods': ['token'],
                    'token': {
                        'id': token
                    }
                },
                'scope': {
                    'service_provider': {
                        'id': self.sp_id
                    }
                }
            }
        }
        resp, saml = self.auth_client.post('auth/OS-FEDERATION/saml2/ecp',
                                           json.dumps(body))
        self.auth_client.expected_success(200, resp.status)

        return etree.XML(saml), self.sp_url

    @testtools.skipUnless(CONF.identity_feature_enabled.federation,
                          "Federated Identity feature not enabled")
    @testtools.skipUnless(CONF.fed_scenario.protocol_id == 'mapped',
                          "Protocol not mapped")
    def test_request_unscoped_token(self):
        self._test_request_unscoped_token()

    @testtools.skipUnless(CONF.identity_feature_enabled.federation,
                          "Federated Identity feature not enabled")
    @testtools.skipUnless(CONF.fed_scenario.protocol_id == 'mapped',
                          "Protocol not mapped")
    def test_request_scoped_token(self):
        self._test_request_scoped_token()



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacProjectEndpointsTests(rbac_base.IdentityV3RbacBaseTests,
                                          metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacProjectEndpointsTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.endpoint_filter_client
        cls.admin_client = cls.os_system_admin
        cls.admin_ef_client = cls.admin_client.endpoint_filter_client

    @classmethod
    def resource_setup(cls):
        super(IdentityV3RbacProjectEndpointsTests, cls).resource_setup()
        cls.project_id = cls.admin_client.projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.projects_client.delete_project,
            project_id=cls.project_id)
        service = cls.admin_client.identity_services_v3_client.create_service(
            type=data_utils.rand_name())['service']
        cls.addClassResourceCleanup(
            cls.admin_client.identity_services_v3_client.delete_service,
            service['id'])
        cls.endpoint_id = cls.admin_client.endpoints_v3_client.create_endpoint(
            interface='public',
            url='http://localhost/foo',
            service_id=service['id'])['endpoint']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.endpoints_v3_client.delete_endpoint,
            endpoint_id=cls.endpoint_id)

    @abc.abstractmethod
    def test_identity_add_endpoint_to_project(self):
        """Test identity:add_endpoint_to_project policy.

        This test must check:
          * whether the persona can allow a project to access an endpoint
        """
        pass

    @abc.abstractmethod
    def test_identity_check_endpoint_in_project(self):
        """Test identity:check_endpoint_in_project policy.

        This test must check:
          * whether the persona can check if a project has access to an
            endpoint
        """
        pass

    @abc.abstractmethod
    def test_identity_list_projects_for_endpoint(self):
        """Test identity:list_projects_for_endpoint policy.

        This test must check:
          * whether the persona can list all projects that have access to an
            endpoint
        """
        pass

    @abc.abstractmethod
    def test_identity_list_endpoints_for_project(self):
        """Test identity:list_endpoints_for_project policy.

        This test must check:
          * whether the persona can list all endpoints to which a project has
            access
        """
        pass

    @abc.abstractmethod
    def test_identity_remove_endpoint_from_project(self):
        """Test identity:remove_endpoint_from_project policy.

        This test must check
          * whether the persona can remove a project's access to an endpoint
        """
        pass


class SystemAdminTests(IdentityV3RbacProjectEndpointsTests):

    credentials = ['system_admin']

    def test_identity_add_endpoint_to_project(self):
        self.do_request('add_endpoint_to_project',
                        expected_status=204,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)
        self.addCleanup(self.admin_ef_client.delete_endpoint_from_project,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)

    def test_identity_check_endpoint_in_project(self):
        self.admin_ef_client.add_endpoint_to_project(
            project_id=self.project_id,
            endpoint_id=self.endpoint_id)
        self.addCleanup(self.admin_ef_client.delete_endpoint_from_project,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)
        self.do_request('check_endpoint_in_project',
                        expected_status=204,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)

    def test_identity_list_projects_for_endpoint(self):
        self.admin_ef_client.add_endpoint_to_project(
            project_id=self.project_id,
            endpoint_id=self.endpoint_id)
        self.addCleanup(self.admin_ef_client.delete_endpoint_from_project,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)
        resp = self.do_request('list_projects_for_endpoint',
                               endpoint_id=self.endpoint_id)
        self.assertIn(self.project_id, [p['id'] for p in resp['projects']])

    def test_identity_list_endpoints_for_project(self):
        self.admin_ef_client.add_endpoint_to_project(
            project_id=self.project_id,
            endpoint_id=self.endpoint_id)
        self.addCleanup(self.admin_ef_client.delete_endpoint_from_project,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)
        resp = self.do_request('list_endpoints_in_project',
                               project_id=self.project_id)
        self.assertIn(self.endpoint_id, [e['id'] for e in resp['endpoints']])

    def test_identity_remove_endpoint_from_project(self):
        self.admin_ef_client.add_endpoint_to_project(
            project_id=self.project_id,
            endpoint_id=self.endpoint_id)
        self.do_request('delete_endpoint_from_project',
                        expected_status=204,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)


class SystemMemberTests(SystemAdminTests):

    credentials = ['system_member', 'system_admin']

    def test_identity_add_endpoint_to_project(self):
        self.do_request('add_endpoint_to_project',
                        expected_status=exceptions.Forbidden,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)

    def test_identity_remove_endpoint_from_project(self):
        self.admin_ef_client.add_endpoint_to_project(
            project_id=self.project_id,
            endpoint_id=self.endpoint_id)
        self.addCleanup(self.admin_ef_client.delete_endpoint_from_project,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)
        self.do_request('delete_endpoint_from_project',
                        expected_status=exceptions.Forbidden,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_check_endpoint_in_project(self):
        self.admin_ef_client.add_endpoint_to_project(
            project_id=self.project_id,
            endpoint_id=self.endpoint_id)
        self.addCleanup(self.admin_ef_client.delete_endpoint_from_project,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)
        self.do_request('check_endpoint_in_project',
                        expected_status=exceptions.Forbidden,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)

    def test_identity_list_projects_for_endpoint(self):
        self.admin_ef_client.add_endpoint_to_project(
            project_id=self.project_id,
            endpoint_id=self.endpoint_id)
        self.addCleanup(self.admin_ef_client.delete_endpoint_from_project,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)
        self.do_request('list_projects_for_endpoint',
                        expected_status=exceptions.Forbidden,
                        endpoint_id=self.endpoint_id)

    def test_identity_list_endpoints_for_project(self):
        self.admin_ef_client.add_endpoint_to_project(
            project_id=self.project_id,
            endpoint_id=self.endpoint_id)
        self.addCleanup(self.admin_ef_client.delete_endpoint_from_project,
                        project_id=self.project_id,
                        endpoint_id=self.endpoint_id)
        self.do_request('list_endpoints_in_project',
                        expected_status=exceptions.Forbidden,
                        project_id=self.project_id)


class DomainManagerTests(DomainAdminTests, base.BaseIdentityTest):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest import clients
from tempest.lib import auth
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacApplicationCredentialTest(
    rbac_base.IdentityV3RbacBaseTests, metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacApplicationCredentialTest, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.application_credentials_client
        cls.admin_client = cls.os_system_admin

    @classmethod
    def setup_user_client(cls, domain_id=None):
        """Set up project user with its own client.

        This is to enable the project user to create its own app cred.

        Returns a client object and the user's ID.
        """
        user_dict = {
            'name': data_utils.rand_name('user'),
            'password': data_utils.rand_password(),
        }
        if domain_id:
            user_dict['domain_id'] = domain_id
        user_id = cls.admin_client.users_v3_client.create_user(
            **user_dict)['user']['id']

        def try_cleanup_user():
            # if domain is cleaned up first, user will already be deleted
            try:
                cls.admin_client.users_v3_client.delete_user(user_id)
            except exceptions.NotFound:
                pass

        cls.addClassResourceCleanup(try_cleanup_user)
        project_id = cls.admin_client.projects_client.create_project(
            data_utils.rand_name())['project']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.projects_client.delete_project, project_id)
        member_role_id = cls.admin_client.roles_v3_client.list_roles(
            name='member')['roles'][0]['id']
        cls.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, member_role_id)
        creds = auth.KeystoneV3Credentials(
            user_id=user_id,
            password=user_dict['password'],
            project_id=project_id)
        auth_provider = clients.get_auth_provider(creds)
        creds = auth_provider.fill_credentials()
        client = clients.Manager(credentials=creds)
        return client, user_id

    def app_cred(self):
        app_cred = {}
        app_cred['name'] = data_utils.rand_name('app_cred')
        return app_cred

    @abc.abstractmethod
    def test_identity_create_application_credential(self):
        """Test identity:create_application_credential policy.

        This test must check:
          * whether the persona can create an application credential for
            themself
          * whether the persona can create an application credential for
            another user
        """
        pass

    @abc.abstractmethod
    def test_identity_get_application_credential(self):
        """Test identity:get_application_credential policy.

        This test must check:
          * whether the persona can get their own application credential
          * whether the persona can get an application credential for another
            user
          * whether the persona can get an application credential for a user in
            another domain (if applicable)
          * whether the persona can get an application credential for a user in
            their own domain (if applicable)
          * whether the persona can get an application credential that does not
            exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_application_credentials(self):
        """Test identity:list_application_credentials policy.

        This test must check:
          * whether the persona can list all application credentials for
            themself
          * whether the persona can list all application credentials for
            another user
          * whether the persona can list application credentials for a user in
            their own domain
          * whether the persona can list application credentials for a user in
            another domain
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_application_credential(self):
        """Test identity:delete_application_credential policy.

        This test must check
          * whether the persona can delete their own application credential
          * whether the persona can delete an application credential for
            another user
          * whether the persona can delete an application credential for a user
            in another domain (if applicable)
          * whether the persona can delete an application credential for a user
            in their own domain (if applicable)
          * whether the persona can delete an application credential that does
            not exist
        """
        pass


class SystemAdminTests(
    IdentityV3RbacApplicationCredentialTest, base.BaseIdentityTest):

    credentials = ['system_admin']

    @classmethod
    def setup_clients(cls):
        super(SystemAdminTests, cls).setup_clients()
        cls.test_user_client, cls.test_user_id = cls.setup_user_client()

    def test_identity_create_application_credential(self):
        # Creating an application credential requires a project ID in the
        # token, therefore system-scoped users cannot create app creds.
        raise self.skipException(
            "Skipping identity:create_application_credential test for "
            "system user")

    def test_identity_get_application_credential(self):
        # Creating an application credential requires a project ID in the
        # token, therefore system-scoped users cannot create app creds, so skip
        # check for showing user's own app creds

        # retrieve other user's app cred
        user_app_cred_client = \
            self.test_user_client.application_credentials_client
        app_cred = user_app_cred_client.create_application_credential(
            user_id=self.test_user_id, **self.app_cred()
        )['application_credential']
        self.addCleanup(
            user_app_cred_client.delete_application_credential,
            self.test_user_id,
            app_cred['id'])
        self.do_request(
            'show_application_credential',
            user_id=self.test_user_id,
            application_credential_id=app_cred['id'])

        # retrieve app cred that does not exist
        self.do_request(
            'show_application_credential',
            expected_status=exceptions.NotFound,
            user_id=self.test_user_id,
            application_credential_id=data_utils.rand_uuid_hex())

    def test_identity_list_application_credentials(self):
        # Creating an application credential requires a project ID in the
        # token, therefore system-scoped users cannot create app creds, so skip
        # check for listing user's own app creds

        # list other user's app creds
        user_app_cred_client = \
            self.test_user_client.application_credentials_client
        app_cred = user_app_cred_client.create_application_credential(
            user_id=self.test_user_id, **self.app_cred()
        )['application_credential']
        self.addCleanup(
            user_app_cred_client.delete_application_credential,
            self.test_user_id,
            app_cred['id'])
        resp = self.do_request(
            'list_application_credentials',
            user_id=self.test_user_id)
        self.assertEqual(
            resp['application_credentials'][0]['id'],
            app_cred['id'])

    def test_identity_delete_application_credential(self):
        # Creating an application credential requires a project ID in the
        # token, therefore system-scoped users cannot create app creds, so skip
        # check for deleting user's own app creds

        # delete other user's app cred
        user_app_cred_client = \
            self.test_user_client.application_credentials_client
        app_cred = user_app_cred_client.create_application_credential(
            user_id=self.test_user_id, **self.app_cred()
        )['application_credential']
        self.do_request(
            'delete_application_credential',
            expected_status=204,
            user_id=self.test_user_id,
            application_credential_id=app_cred['id'])

        # delete app cred that does not exist
        self.do_request(
            'delete_application_credential',
            expected_status=exceptions.NotFound,
            user_id=self.test_user_id,
            application_credential_id=data_utils.rand_uuid_hex())


class SystemMemberTests(SystemAdminTests):

    credentials = ['system_member', 'system_admin']

    def test_identity_delete_application_credential(self):
        # Creating an application credential requires a project ID in the
        # token, therefore system-scoped users cannot create app creds, so skip
        # check for deleting user's own app creds

        # delete other user's app cred
        user_app_cred_client = \
            self.test_user_client.application_credentials_client
        app_cred = user_app_cred_client.create_application_credential(
            user_id=self.test_user_id, **self.app_cred()
        )['application_credential']
        self.addCleanup(
            user_app_cred_client.delete_application_credential,
            self.test_user_id,
            app_cred['id'])
        self.do_request(
            'delete_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_id,
            application_credential_id=app_cred['id'])

        # delete app cred that does not exist
        self.do_request(
            'delete_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_id,
            application_credential_id=data_utils.rand_uuid_hex())


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(
    IdentityV3RbacApplicationCredentialTest, base.BaseIdentityTest):

    # Domain admins cannot create their own app creds (app creds can only be
    # scoped to projects) and domain admins have no special privileges over the
    # app creds own by users in their domains.

    credentials = ['domain_admin', 'system_admin']

    @classmethod
    def setup_clients(cls):
        super(DomainAdminTests, cls).setup_clients()
        own_domain_id = cls.persona.credentials.domain_id
        cls.test_client_1, cls.test_user_1 = cls.setup_user_client(
            domain_id=own_domain_id)

    def setUp(self):
        super(DomainAdminTests, self).setUp()
        self.other_domain_id = self.admin_client.domains_client.create_domain(
            name=data_utils.rand_name())['domain']['id']
        self.addCleanup(self.admin_client.domains_client.delete_domain,
                        self.other_domain_id)
        self.addCleanup(self.admin_client.domains_client.update_domain,
                        domain_id=self.other_domain_id, enabled=False)
        self.test_client_2, self.test_user_2 = self.setup_user_client(
            domain_id=self.other_domain_id)
        client = self.test_client_1.application_credentials_client
        self.app_cred_1 = client.create_application_credential(
            user_id=self.test_user_1, **self.app_cred()
        )['application_credential']
        self.addCleanup(
            client.delete_application_credential,
            self.test_user_1,
            self.app_cred_1['id'])
        client = self.test_client_2.application_credentials_client
        self.app_cred_2 = client.create_application_credential(
            user_id=self.test_user_2, **self.app_cred()
        )['application_credential']
        self.addCleanup(
            client.delete_application_credential,
            self.test_user_2,
            self.app_cred_2['id'])

    def test_identity_create_application_credential(self):
        # Creating an application credential requires a project ID in the
        # token, therefore system-scoped users cannot create app creds.
        raise self.skipException(
            "Skipping identity:create_application_credential test for "
            "domain user")

    def test_identity_get_application_credential(self):
        # Creating an application credential requires a project ID in the
        # token, therefore domain-scoped users cannot create app creds, so skip
        # check for showing user's own app creds

        # accessing application credentials should be forbidden no matter
        # whether the owner is in the domain or outside of it

        # retrieve app cred from user in own domain
        self.do_request(
            'show_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_1,
            application_credential_id=self.app_cred_1['id'])

        # retrieve app cred from user in other domain
        self.do_request(
            'show_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_2,
            application_credential_id=self.app_cred_2['id'])

        # retrieve app cred that does not exist
        self.do_request(
            'show_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_1,
            application_credential_id=data_utils.rand_uuid_hex())

    def test_identity_list_application_credentials(self):
        # Creating an application credential requires a project ID in the
        # token, therefore domain-scoped users cannot create app creds, so skip
        # check for listing user's own app creds

        # listing application credentials should be forbidden no matter
        # whether the owner is in the domain or outside of it

        # list app creds from user in own domain
        self.do_request(
            'list_application_credentials',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_1)

        # list app creds from user in other domain
        self.do_request(
            'list_application_credentials',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_2)

    def test_identity_delete_application_credential(self):
        # Creating an application credential requires a project ID in the
        # token, therefore domain-scoped users cannot create app creds, so skip
        # check for deleting user's own app creds

        # deleting application credentials should be forbidden no matter
        # whether the owner is in the domain or outside of it

        # delete app cred from user in own domain
        self.do_request(
            'delete_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_1,
            application_credential_id=self.app_cred_1['id'])

        # delete app cred from user in other domain
        self.do_request(
            'delete_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_2,
            application_credential_id=self.app_cred_2['id'])

        # delete app cred that does not exist
        self.do_request(
            'delete_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_1,
            application_credential_id=data_utils.rand_uuid_hex())


class DomainManagerTests(DomainAdminTests):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainAdminTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(IdentityV3RbacApplicationCredentialTest,
                          base.BaseIdentityTest):

    credentials = ['project_manager', 'system_admin']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.test_user_client, cls.test_user_id = cls.setup_user_client()

    def test_identity_create_application_credential(self):
        # user can create their own app cred
        user_id = self.persona.credentials.user_id
        resp = self.do_request(
            'create_application_credential',
            expected_status=201,
            user_id=user_id,
            **self.app_cred())['application_credential']
        self.addCleanup(
            self.client.delete_application_credential,
            user_id, resp['id'])

        # user cannot create app cred for another user
        user_id = self.test_user_id
        self.do_request(
            'create_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=user_id,
            **self.app_cred())

    def test_identity_get_application_credential(self):
        # user can retrieve their own app cred
        user_id = self.persona.credentials.user_id
        app_cred = self.client.create_application_credential(
            user_id=user_id, **self.app_cred())['application_credential']
        self.addCleanup(
            self.client.delete_application_credential,
            user_id=user_id, application_credential_id=app_cred['id'])
        self.do_request(
            'show_application_credential',
            user_id=user_id, application_credential_id=app_cred['id'])

        # retrieving non-existent app cred for self should return 404
        self.do_request(
            'show_application_credential',
            expected_status=exceptions.NotFound,
            user_id=user_id,
            application_credential_id=data_utils.rand_uuid_hex())

        # user cannot retrieve another user's app cred by using the victim's
        # user ID in the request or by trying to bypass the user ownership
        # check by crafting a path the the attacker's user ID
        user_id = self.test_user_id
        client = self.test_user_client.application_credentials_client
        app_cred = client.create_application_credential(
            user_id=user_id, **self.app_cred())['application_credential']
        self.addCleanup(
            client.delete_application_credential,
            user_id=user_id, application_credential_id=app_cred['id'])
        self.do_request(
            'show_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.persona.credentials.user_id,
            application_credential_id=app_cred['id'])
        self.do_request(
            'show_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=user_id, application_credential_id=app_cred['id'])

        # retrieving non-existent app cred for another user should return 403
        self.do_request(
            'show_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=user_id,
            application_credential_id=data_utils.rand_uuid_hex())

    def test_identity_list_application_credentials(self):
        # user can list their own app creds
        user_id = self.persona.credentials.user_id
        app_cred = self.client.create_application_credential(
            user_id=user_id, **self.app_cred())['application_credential']
        self.addCleanup(
            self.client.delete_application_credential,
            user_id=user_id, application_credential_id=app_cred['id'])
        self.do_request(
            'list_application_credentials', user_id=user_id)

        # user cannot list another user's app creds
        user_id = self.test_user_id
        client = self.test_user_client.application_credentials_client
        app_cred = client.create_application_credential(
            user_id=user_id, **self.app_cred())['application_credential']
        self.addCleanup(
            client.delete_application_credential,
            user_id=user_id, application_credential_id=app_cred['id'])
        self.do_request(
            'list_application_credentials',
            expected_status=exceptions.Forbidden, user_id=user_id)

    def test_identity_delete_application_credential(self):
        # user can delete their own app cred
        user_id = self.persona.credentials.user_id
        app_cred = self.client.create_application_credential(
            user_id=user_id, **self.app_cred())['application_credential']
        self.do_request(
            'delete_application_credential',
            expected_status=204,
            user_id=user_id, application_credential_id=app_cred['id'])

        # deleting non-existent app cred for self should return 404
        self.do_request(
            'delete_application_credential',
            expected_status=exceptions.NotFound,
            user_id=user_id,
            application_credential_id=data_utils.rand_uuid_hex())

        # user cannot delete another user's app cred by using the victim's
        # user ID in the request or by trying to bypass the user ownership
        # check by crafting a path the the attacker's user ID
        user_id = self.test_user_id
        client = self.test_user_client.application_credentials_client
        app_cred = client.create_application_credential(
            user_id=user_id, **self.app_cred())['application_credential']
        self.addCleanup(
            client.delete_application_credential,
            user_id=user_id, application_credential_id=app_cred['id'])
        self.do_request(
            'delete_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.persona.credentials.user_id,
            application_credential_id=app_cred['id'])
        self.do_request(
            'delete_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=user_id, application_credential_id=app_cred['id'])

        # deleting non-existent app cred for another user should return 403
        self.do_request(
            'delete_application_credential',
            expected_status=exceptions.Forbidden,
            user_id=user_id,
            application_credential_id=data_utils.rand_uuid_hex())


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest import clients
from tempest.lib import auth
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacCredentialTest(rbac_base.IdentityV3RbacBaseTests,
                                   metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacCredentialTest, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.credentials_client
        cls.admin_client = cls.os_system_admin
        cls.admin_credentials_client = cls.admin_client.credentials_client

        # personas in own or other domains
        own_domain_id = cls.persona.credentials.domain_id
        cls.test_client_1, cls.test_user_1 = cls.setup_user_client(
            domain_id=own_domain_id)
        cls.other_domain_id = cls.admin_client.domains_client.create_domain(
            name=data_utils.rand_name())['domain']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.domains_client.delete_domain, cls.other_domain_id)
        cls.addClassResourceCleanup(
            cls.admin_client.domains_client.update_domain,
            domain_id=cls.other_domain_id, enabled=False)
        cls.test_client_2, cls.test_user_2 = cls.setup_user_client(
            domain_id=cls.other_domain_id)

    @classmethod
    def setup_user_client(cls, domain_id=None):
        """Set up project user with its own client.

        This is to enable the project user to create its own credential.

        Returns a client object and the user's ID.
        """
        user_dict = {
            'name': data_utils.rand_name('user'),
            'password': data_utils.rand_password(),
        }
        if domain_id:
            user_dict['domain_id'] = domain_id
        user_id = cls.admin_client.users_v3_client.create_user(
            **user_dict)['user']['id']

        def try_cleanup_user():
            # if domain is cleaned up first, user will already be deleted
            try:
                cls.admin_client.users_v3_client.delete_user(user_id)
            except exceptions.NotFound:
                pass

        cls.addClassResourceCleanup(try_cleanup_user)
        project_id = cls.admin_client.projects_client.create_project(
            data_utils.rand_name())['project']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.projects_client.delete_project, project_id)
        member_role_id = cls.admin_client.roles_v3_client.list_roles(
            name='member')['roles'][0]['id']
        cls.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, member_role_id)
        creds = auth.KeystoneV3Credentials(
            user_id=user_id,
            password=user_dict['password'],
            project_id=project_id)
        auth_provider = clients.get_auth_provider(creds)
        creds = auth_provider.fill_credentials()
        client = clients.Manager(credentials=creds)
        return client, user_id

    def credential(self, user_id):
        cred = {
            'blob': data_utils.rand_uuid_hex(),
            'type': data_utils.rand_uuid_hex(),
            'user_id': user_id,
        }
        return cred

    @abc.abstractmethod
    def test_identity_create_credential(self):
        """Test identity:create_credential policy.

        This test must check:
          * whether the persona can create a credential for themself
          * whether the persona can create acredential for another user in
            their own domain
          * whether the persona can create acredential for another user in
            another domain
        """
        pass

    @abc.abstractmethod
    def test_identity_get_credential(self):
        """Test identity:get_credential policy.

        This test must check:
          * whether the persona can get their own credential
          * whether the persona can get a credential for a user in another
            domain
          * whether the persona can get a credential for a user in their own
            domain
          * whether the persona can get a credential that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_credentials(self):
        """Test identity:list_credentials policy.

        This test must check:
          * whether the persona can list all credentials for themself
          * whether the persona can list credentials for a user in their own
            domain
          * whether the persona can list credentials for a user in another
            domain
        """
        pass

    @abc.abstractmethod
    def test_identity_update_credential(self):
        """Test identity:update_credential policy.

        This test must check:
          * whether the persona can update their own credential
          * whether the persona can update a credential for a user in another
            domain
          * whether the persona can update a credential for a user in their own
            domain
          * whether the persona can update a credential that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_credential(self):
        """Test identity:delete_credential policy.

        This test must check
          * whether the persona can delete their own credential
          * whether the persona can delete a credential for a user in another
            domain
          * whether the persona can delete a credential for a user in their own
            domain
          * whether the persona can delete a credential that does not exist
        """
        pass


class SystemAdminTests(IdentityV3RbacCredentialTest, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_credential(self):
        # user can create their own credential
        user_id = self.persona.credentials.user_id
        resp = self.do_request(
            'create_credential',
            expected_status=201,
            **self.credential(user_id=user_id))['credential']
        self.addCleanup(self.client.delete_credential, resp['id'])
        # user can create credential for other user in own domain
        resp = self.do_request(
            'create_credential',
            expected_status=201,
            **self.credential(user_id=self.test_user_1))['credential']
        self.addCleanup(self.client.delete_credential, resp['id'])
        # user can create credential for other user in other domain
        resp = self.do_request(
            'create_credential',
            expected_status=201,
            **self.credential(user_id=self.test_user_2))['credential']
        self.addCleanup(self.client.delete_credential, resp['id'])

    def test_identity_get_credential(self):
        # user can get their own credential, credential for user in own domain,
        # or credential for user in other domain
        user_id = self.persona.credentials.user_id
        for u in [user_id, self.test_user_1, self.test_user_2]:
            cred = self.admin_credentials_client.create_credential(
                **self.credential(user_id=u))['credential']
            self.addCleanup(
                self.admin_credentials_client.delete_credential, cred['id'])
            self.do_request('show_credential', credential_id=cred['id'])
        # non-existent credential is Not Found
        self.do_request(
            'show_credential',
            expected_status=exceptions.NotFound,
            credential_id=data_utils.rand_uuid_hex())

    def test_identity_list_credentials(self):
        # user can list their own credentials, credentials for user in own
        # domain, or credentials for user in other domain
        user_id = self.persona.credentials.user_id
        for u in [user_id, self.test_user_1, self.test_user_2]:
            cred = self.admin_credentials_client.create_credential(
                **self.credential(user_id=u))['credential']
            self.addCleanup(
                self.admin_credentials_client.delete_credential, cred['id'])
            resp = self.do_request('list_credentials')['credentials']
            self.assertIn(cred['id'], [c['id'] for c in resp])

    def test_identity_update_credential(self):
        # user can update their own credential, credential for user in own
        # domain, or credential for user in other domain
        user_id = self.persona.credentials.user_id
        for u in [user_id, self.test_user_1, self.test_user_2]:
            cred = self.credential(user_id=u)
            resp = self.client.create_credential(**cred)['credential']
            self.addCleanup(self.client.delete_credential, resp['id'])
            cred['blob'] = data_utils.rand_uuid_hex()
            self.do_request(
                'update_credential', credential_id=resp['id'], **cred)
        # non-existent credential is Not Found
        self.do_request(
            'update_credential',
            expected_status=exceptions.NotFound,
            credential_id=data_utils.rand_uuid_hex(),
            **self.credential(user_id=self.test_user_2))

    def test_identity_delete_credential(self):
        # user can delete their own credential, credential for user in own
        # domain, or credential for user in other domain
        user_id = self.persona.credentials.user_id
        for u in [user_id, self.test_user_1, self.test_user_2]:
            cred = self.credential(user_id=u)
            resp = self.client.create_credential(**cred)['credential']
            self.do_request(
                'delete_credential',
                expected_status=204,
                credential_id=resp['id'])
        # non-existent credential is Not Found
        self.do_request(
            'delete_credential',
            expected_status=exceptions.NotFound,
            credential_id=data_utils.rand_uuid_hex())


class SystemMemberTests(SystemAdminTests):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_credential(self):
        # user can create their own credential
        user_id = self.persona.credentials.user_id
        resp = self.do_request(
            'create_credential',
            expected_status=201,
            **self.credential(user_id=user_id))['credential']
        self.addCleanup(self.client.delete_credential, resp['id'])
        # user cannot create credential for other user
        for u in [self.test_user_1, self.test_user_2]:
            self.do_request(
                'create_credential',
                expected_status=exceptions.Forbidden,
                **self.credential(user_id=u))

    def test_identity_update_credential(self):
        # user can update their own credential
        user_id = self.persona.credentials.user_id
        cred = self.credential(user_id=user_id)
        resp = self.admin_credentials_client.create_credential(
            **cred)['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_credential, resp['id'])
        cred['blob'] = data_utils.rand_uuid_hex()
        self.do_request(
            'update_credential',
            credential_id=resp['id'], **cred)
        # user cannot update credential for other user
        for u in [self.test_user_1, self.test_user_2]:
            cred = self.credential(user_id=u)
            resp = self.admin_credentials_client.create_credential(
                **cred)['credential']
            self.addCleanup(
                self.admin_credentials_client.delete_credential, resp['id'])
            cred['blob'] = data_utils.rand_uuid_hex()
            self.do_request(
                'update_credential',
                expected_status=exceptions.Forbidden,
                credential_id=resp['id'], **cred)
        # non-existent credential is Forbidden
        self.do_request(
            'update_credential',
            expected_status=exceptions.Forbidden,
            credential_id=data_utils.rand_uuid_hex(),
            **self.credential(user_id=self.test_user_2))

    def test_identity_delete_credential(self):
        # user can delete their own credential
        user_id = self.persona.credentials.user_id
        cred = self.credential(user_id=user_id)
        resp = self.admin_credentials_client.create_credential(
            **cred)['credential']
        self.do_request(
            'delete_credential',
            expected_status=204,
            credential_id=resp['id'])
        # user cannot delete credential for other user
        for u in [self.test_user_1, self.test_user_2]:
            cred = self.credential(user_id=u)
            resp = self.admin_credentials_client.create_credential(
                **cred)['credential']
            self.addCleanup(
                self.admin_credentials_client.delete_credential, resp['id'])
            self.do_request(
                'delete_credential',
                expected_status=exceptions.Forbidden,
                credential_id=resp['id'])
        # non-existent credential is Forbidden
        self.do_request(
            'delete_credential',
            expected_status=exceptions.Forbidden,
            credential_id=data_utils.rand_uuid_hex())


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemAdminTests):

    credentials = ['domain_admin', 'system_admin']


class DomainManagerTests(SystemReaderTests):

    credentials = ['domain_manager', 'system_admin']

    def test_identity_get_credential(self):
        # user can get their own credential
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_credential(
            **self.credential(user_id=user_id))['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_credential, cred['id'])
        self.do_request('show_credential', credential_id=cred['id'])
        # user cannot get credential for another user
        for u in [self.test_user_1, self.test_user_2]:
            cred = self.admin_credentials_client.create_credential(
                **self.credential(user_id=u))['credential']
            self.addCleanup(
                self.admin_credentials_client.delete_credential, cred['id'])
            self.do_request(
                'show_credential',
                expected_status=exceptions.Forbidden,
                credential_id=cred['id'])
        # non-existent credential is Forbidden
        self.do_request(
            'show_credential',
            expected_status=exceptions.Forbidden,
            credential_id=data_utils.rand_uuid_hex())

    def test_identity_list_credentials(self):
        # user can list their own credentials
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_credential(
            **self.credential(user_id=user_id))['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_credential, cred['id'])
        resp = self.do_request('list_credentials')['credentials']
        self.assertIn(cred['id'], [c['id'] for c in resp])
        # user cannot list credentials for other users
        for u in [self.test_user_1, self.test_user_2]:
            cred = self.admin_credentials_client.create_credential(
                **self.credential(user_id=u))['credential']
            self.addCleanup(
                self.admin_credentials_client.delete_credential, cred['id'])
            resp = self.do_request('list_credentials')['credentials']
            self.assertNotIn(cred['id'], [c['id'] for c in resp])


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacRoleTest(rbac_base.IdentityV3RbacBaseTests,
                             metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacRoleTest, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.roles_v3_client
        cls.admin_client = cls.os_system_admin
        cls.admin_roles_client = cls.admin_client.roles_v3_client

    @classmethod
    def resource_setup(cls):
        super(IdentityV3RbacRoleTest, cls).resource_setup()
        cls.own_domain = cls.persona.credentials.domain_id
        cls.domain_id = cls.admin_client.domains_client.create_domain(
            name=data_utils.rand_name('domain'))['domain']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.domains_client.delete_domain,
            cls.domain_id)
        cls.addClassResourceCleanup(
            cls.admin_client.domains_client.update_domain,
            cls.domain_id,
            enabled=False)
        cls.member_role_id = cls.admin_client.roles_v3_client.list_roles(
            name='member')['roles'][0]['id']

    def role(self, domain_id=None):
        role = {}
        name = data_utils.rand_name('role')
        role['name'] = name
        if domain_id:
            role['domain_id'] = domain_id
        return role

    @abc.abstractmethod
    def test_identity_create_role(self):
        """Test identity:create_role policy.

        This test must check:
          * whether the persona can create a role
        """
        pass

    @abc.abstractmethod
    def test_identity_get_role(self):
        """Test identity:get_role policy.

        This test must check:
          * whether the persona can get a role
          * whether the persona can get a role that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_roles(self):
        """Test identity:list_roles policy.

        This test must check:
          * whether the persona can list roles
        """
        pass

    @abc.abstractmethod
    def test_identity_update_role(self):
        """Test identity:update_role policy.

        This test must check:
          * whether the persona can update a role
          * whether the persona can update a role that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_role(self):
        """Test identity:delete_role policy.

        This test must check
          * whether the persona can delete a role
          * whether the persona can delete a role that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_create_domain_role(self):
        """Test identity:create_domain_role policy.

        This test must check:
          * whether the persona can create a domain role in their own domain
          * whether the persona can create a domain role in another domain
        """
        pass

    @abc.abstractmethod
    def test_identity_get_domain_role(self):
        """Test identity:get_domain_role policy.

        This test must check:
          * whether the persona can get a domain role in their own domain
          * whether the persona can get a domain role in another domain
          * whether the persona can get a domain role that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_domain_roles(self):
        """Test identity:list_domain_roles policy.

        This test must check:
          * whether the persona can list domain roles for their own domain
          * whether the persona can list domain roles for another domain
        """
        pass

    @abc.abstractmethod
    def test_identity_update_domain_role(self):
        """Test identity:update_domain_role policy.

        This test must check:
          * whether the persona can update a domain role for their own domain
          * whether the persona can update a domain role for another domain
          * whether the persona can update a domain role that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_domain_role(self):
        """Test identity:delete_domain_role policy.

        This test must check
          * whether the persona can delete a domain role for their own domain
          * whether the persona can delete a domain role for another domain
          * whether the persona can delete a domain role that does not exist
        """
        pass


class SystemAdminTests(IdentityV3RbacRoleTest, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_role(self):
        # user can create role
        resp = self.do_request('create_role',
                               expected_status=201,
                               **self.role())
        self.addCleanup(self.admin_roles_client.delete_role,
                        resp['role']['id'])

    def test_identity_get_role(self):
        # user can get role
        role = self.admin_roles_client.create_role(**self.role())['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('show_role', role_id=role['id'])
        # user gets a 404 for nonexistent role
        self.do_request('show_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex())

    def test_identity_list_roles(self):
        # user can list roles
        role = self.admin_roles_client.create_role(**self.role())['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('list_roles')

    def test_identity_update_role(self):
        # user can update role
        role = self.admin_roles_client.create_role(**self.role())['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('update_role',
                        role_id=role['id'],
                        description=data_utils.arbitrary_string())
        # user gets a 404 for nonexistent role
        self.do_request('update_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_role(self):
        # user can delete role
        role = self.admin_roles_client.create_role(**self.role())['role']
        self.do_request('delete_role', expected_status=204, role_id=role['id'])
        # user gets a 404 for nonexistent role
        self.do_request('delete_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex())

    def test_identity_create_domain_role(self):
        # user can create domain role
        resp = self.do_request('create_role',
                               expected_status=201,
                               **self.role(domain_id=self.domain_id))
        self.addCleanup(self.admin_roles_client.delete_role,
                        resp['role']['id'])

    def test_identity_get_domain_role(self):
        # user can get domain role
        role = self.admin_roles_client.create_role(
            **self.role(domain_id=self.domain_id))['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('show_role', role_id=role['id'])
        # user gets a 404 for nonexistent domain role
        self.do_request('show_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex())

    def test_identity_list_domain_roles(self):
        # user can list domain roles
        role = self.admin_roles_client.create_role(
            **self.role(domain_id=self.domain_id))['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('list_roles', domain_id=self.domain_id)

    def test_identity_update_domain_role(self):
        # user can update domain role
        role = self.admin_roles_client.create_role(
            **self.role(domain_id=self.domain_id))['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('update_role',
                        role_id=role['id'],
                        description=data_utils.arbitrary_string())
        # user gets a 404 for nonexistent domain role
        self.do_request('update_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_domain_role(self):
        # user can delete role in other domain
        role = self.admin_roles_client.create_role(
            **self.role(domain_id=self.domain_id))['role']
        self.do_request('delete_role', expected_status=204, role_id=role['id'])
        # user gets a 404 for nonexistent role
        self.do_request('delete_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex())


class SystemMemberTests(SystemAdminTests):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_role(self):
        # user cannot create role
        self.do_request('create_role',
                        expected_status=exceptions.Forbidden,
                        **self.role())

    def test_identity_update_role(self):
        # user cannot update role
        role = self.admin_roles_client.create_role(
            **self.role())['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('update_role', expected_status=exceptions.Forbidden,
                        role_id=role['id'],
                        description=data_utils.arbitrary_string())
        # user gets a 404 for nonexistent role
        self.do_request('update_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_role(self):
        # user can delete role
        role = self.admin_roles_client.create_role(
            **self.role())['role']
        self.do_request('delete_role', expected_status=exceptions.Forbidden,
                        role_id=role['id'])
        # user gets a 404 for nonexistent domain role
        self.do_request('delete_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex())

    def test_identity_create_domain_role(self):
        # user cannot create domain role
        self.do_request('create_role',
                        expected_status=exceptions.Forbidden,
                        **self.role(domain_id=self.domain_id))

    def test_identity_update_domain_role(self):
        # user cannot update domain role
        role = self.admin_roles_client.create_role(
            **self.role(domain_id=self.domain_id))['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('update_role', expected_status=exceptions.Forbidden,
                        role_id=role['id'],
                        description=data_utils.arbitrary_string())
        # user gets a 404 for nonexistent domain role
        self.do_request('update_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_domain_role(self):
        # user cannot delete domain role
        role = self.admin_roles_client.create_role(
            **self.role(domain_id=self.domain_id))['role']
        self.do_request('delete_role', expected_status=exceptions.Forbidden,
                        role_id=role['id'])
        # user gets a 404 for nonexistent domain role
        self.do_request('delete_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex())


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_domain_role(self):
        # user cannot get domain role in own domain
        role = self.admin_roles_client.create_role(
            **self.role(domain_id=self.own_domain))['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('show_role', expected_status=exceptions.Forbidden,
                        role_id=role['id'])
        # user gets a 404 for nonexistent domain role
        self.do_request('show_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex())

    def test_identity_list_domain_roles(self):
        # user cannot list domain roles in own domain
        role = self.admin_roles_client.create_role(
            **self.role(domain_id=self.own_domain))['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('list_roles', expected_status=exceptions.Forbidden,
                        domain_id=self.persona.credentials.domain_id)
        # user cannot get domain role in other domain
        role = self.admin_roles_client.create_role(
            **self.role(domain_id=self.domain_id))['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('list_roles', expected_status=exceptions.Forbidden,
                        domain_id=self.domain_id)


class DomainManagerTests(DomainAdminTests):

    credentials = ['domain_manager', 'system_admin']

    def test_identity_get_role(self):
        # user can get role that is part of the domain_managed_target_role
        # list for domain managers, which per default includes "member"
        self.do_request('show_role', role_id=self.member_role_id)
        # user cannot get arbitrary global role not part of the
        # domain_managed_target_role list
        role = self.admin_roles_client.create_role(
            **self.role())['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('show_role', expected_status=exceptions.Forbidden,
                        role_id=role['id'])
        # user gets a 404 for nonexistent role
        self.do_request('show_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex())


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']

    def test_identity_get_role(self):
        # user cannot get role
        role = self.admin_roles_client.create_role(
            **self.role())['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('show_role', expected_status=exceptions.Forbidden,
                        role_id=role['id'])
        # user gets a 404 for nonexistent role
        self.do_request('show_role', expected_status=exceptions.NotFound,
                        role_id=data_utils.rand_uuid_hex())

    def test_identity_list_roles(self):
        # user cannot list roles
        role = self.admin_roles_client.create_role(**self.role())['role']
        self.addCleanup(self.admin_roles_client.delete_role, role['id'])
        self.do_request('list_roles', expected_status=exceptions.Forbidden)


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainMemberTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC #
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin import clients
from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacIdentityProviderTests(rbac_base.IdentityV3RbacBaseTests,
                                          metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacIdentityProviderTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.keystone_manager = clients.Manager(cls.persona.credentials)
        persona_mgr = clients.Manager(cls.persona.credentials)
        cls.client = persona_mgr.identity_providers_client
        cls.admin_client = cls.os_system_admin
        admin_mgr = clients.Manager(cls.admin_client.credentials)
        cls.admin_idp_client = admin_mgr.identity_providers_client

    @abc.abstractmethod
    def test_identity_create_identity_provider(self):
        """Test identity:create_identity_provider policy.

        This test must check:
          * whether the persona can create an identity provider
        """
        pass

    @abc.abstractmethod
    def test_identity_get_identity_provider(self):
        """Test identity:get_identity_provider policy.

        This test must check:
          * whether the persona can get an identity provider
          * whether the persona can get an identity provider that does not
            exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_identity_providers(self):
        """Test identity:list_identity_providers policy.

        This test must check:
          * whether the persona can list all identity providers
        """
        pass

    @abc.abstractmethod
    def test_identity_update_identity_provider(self):
        """Test identity:update_identity_provider policy.

        This test must check:
          * whether the persona can update an identity provider
          * whether the persona can update an identity provider that does not
            exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_identity_provider(self):
        """Test identity:delete_identity_provider policy.

        This test must check
          * whether the persona can delete an identity provider
          * whether the persona can delete an identity provider that does not
            exist
        """
        pass


class SystemAdminTests(IdentityV3RbacIdentityProviderTests,
                       base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_identity_provider(self):
        idp = self.do_request(
            'create_identity_provider', expected_status=201,
            idp_id=data_utils.rand_name()
        )['identity_provider']
        self.addCleanup(
            self.admin_client.domains_client.delete_domain, idp['domain_id'])
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            idp['domain_id'], enabled=False)
        self.addCleanup(
            self.admin_idp_client.delete_identity_provider, idp['id'])

    def test_identity_get_identity_provider(self):
        idp = self.admin_idp_client.create_identity_provider(
            idp_id=data_utils.rand_name())['identity_provider']
        self.addCleanup(
            self.admin_client.domains_client.delete_domain, idp['domain_id'])
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            idp['domain_id'], enabled=False)
        self.addCleanup(
            self.admin_idp_client.delete_identity_provider, idp['id'])
        self.do_request('show_identity_provider', idp_id=idp['id'])
        # user gets a 404 for nonexistent idp
        self.do_request('show_identity_provider',
                        expected_status=exceptions.NotFound,
                        idp_id=data_utils.rand_uuid_hex())

    def test_identity_list_identity_providers(self):
        idp = self.admin_idp_client.create_identity_provider(
            idp_id=data_utils.rand_name())['identity_provider']
        self.addCleanup(
            self.admin_client.domains_client.delete_domain, idp['domain_id'])
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            idp['domain_id'], enabled=False)
        self.addCleanup(
            self.admin_idp_client.delete_identity_provider, idp['id'])
        resp = self.do_request('list_identity_providers')
        self.assertIn(idp['id'], [i['id'] for i in resp['identity_providers']])

    def test_identity_update_identity_provider(self):
        idp = self.admin_idp_client.create_identity_provider(
            idp_id=data_utils.rand_name())['identity_provider']
        self.addCleanup(
            self.admin_client.domains_client.delete_domain, idp['domain_id'])
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            idp['domain_id'], enabled=False)
        self.addCleanup(
            self.admin_idp_client.delete_identity_provider, idp['id'])
        self.do_request('update_identity_provider',
                        idp_id=idp['id'],
                        description=data_utils.arbitrary_string())
        # user gets a 404 for nonexistent idp
        self.do_request('update_identity_provider',
                        expected_status=exceptions.NotFound,
                        idp_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_identity_provider(self):
        idp = self.admin_idp_client.create_identity_provider(
            idp_id=data_utils.rand_name())['identity_provider']
        self.addCleanup(
            self.admin_client.domains_client.delete_domain, idp['domain_id'])
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            idp['domain_id'], enabled=False)
        self.do_request('delete_identity_provider', expected_status=204,
                        idp_id=idp['id'])
        # user gets a 404 for nonexistent idp
        self.do_request('delete_identity_provider',
                        expected_status=exceptions.NotFound,
                        idp_id=data_utils.rand_uuid_hex())


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_identity_provider(self):
        self.do_request('create_identity_provider',
                        expected_status=exceptions.Forbidden,
                        idp_id=data_utils.rand_name())

    def test_identity_update_identity_provider(self):
        idp = self.admin_idp_client.create_identity_provider(
            idp_id=data_utils.rand_name())['identity_provider']
        self.addCleanup(
            self.admin_client.domains_client.delete_domain, idp['domain_id'])
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            idp['domain_id'], enabled=False)
        self.addCleanup(
            self.admin_idp_client.delete_identity_provider, idp['id'])
        self.do_request('update_identity_provider',
                        expected_status=exceptions.Forbidden,
                        idp_id=idp['id'],
                        description=data_utils.arbitrary_string())
        # user gets a 403 for nonexistent idp
        self.do_request('update_identity_provider',
                        expected_status=exceptions.Forbidden,
                        idp_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_identity_provider(self):
        idp = self.admin_idp_client.create_identity_provider(
            idp_id=data_utils.rand_name())['identity_provider']
        self.addCleanup(
            self.admin_client.domains_client.delete_domain, idp['domain_id'])
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            idp['domain_id'], enabled=False)
        self.addCleanup(
            self.admin_idp_client.delete_identity_provider, idp['id'])
        self.do_request('delete_identity_provider',
                        expected_status=exceptions.Forbidden,
                        idp_id=idp['id'])
        # user gets a 403 for nonexistent idp
        self.do_request('delete_identity_provider',
                        expected_status=exceptions.Forbidden,
                        idp_id=data_utils.rand_uuid_hex())


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_identity_provider(self):
        idp = self.admin_idp_client.create_identity_provider(
            idp_id=data_utils.rand_name())['identity_provider']
        self.addCleanup(
            self.admin_client.domains_client.delete_domain, idp['domain_id'])
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            idp['domain_id'], enabled=False)
        self.addCleanup(
            self.admin_idp_client.delete_identity_provider, idp['id'])
        self.do_request('show_identity_provider',
                        expected_status=exceptions.Forbidden,
                        idp_id=idp['id'])
        # user gets a 403 for nonexistent idp
        self.do_request('show_identity_provider',
                        expected_status=exceptions.Forbidden,
                        idp_id=data_utils.rand_uuid_hex())

    def test_identity_list_identity_providers(self):
        idp = self.admin_idp_client.create_identity_provider(
            idp_id=data_utils.rand_name())['identity_provider']
        self.addCleanup(
            self.admin_client.domains_client.delete_domain, idp['domain_id'])
        self.addCleanup(
            self.admin_client.domains_client.update_domain,
            idp['domain_id'], enabled=False)
        self.addCleanup(
            self.admin_idp_client.delete_identity_provider, idp['id'])
        self.do_request('list_identity_providers',
                        expected_status=exceptions.Forbidden)


class DomainManagerTests(DomainAdminTests, base.BaseIdentityTest):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from keystone_tempest_plugin.services.identity import clients


class ServiceProvidersClient(clients.Federation):

    subpath_suffix = 'service_providers'

    def create_service_provider(self, sp_id, **kwargs):
        """Create a service provider.

        :param str sp_id: The ID to be used to create the Service Provider.
        :param kwargs: Extra attributes. Mandatory: auth_url (str) and sp_url
                       (str). Optional: description (str), enabled (boolean)
                       and relay_state_prefix (str).
        """
        put_body = json.dumps({'service_provider': kwargs})
        return self._put(sp_id, put_body)

    def list_service_providers(self):
        """List the service providers."""
        return self._get()

    def show_service_provider(self, sp_id):
        """Get a service provider."""
        return self._get(sp_id)

    def delete_service_provider(self, sp_id):
        """Delete a service provider."""
        return self._delete(sp_id)

    def update_service_provider(self, sp_id, **kwargs):
        """Update a service provider.

        :param str sp_id: The ID of the Service Provider to be updated.
        :param kwargs: All attributes to be updated: auth_url (str) and sp_url
                       (str), description (str), enabled (boolean) and
                       relay_state_prefix (str).
        """
        patch_body = json.dumps({'service_provider': kwargs})
        return self._patch(sp_id, patch_body)

    def get_service_providers_in_token(self):
        """Get the service providers list present in the token.

        Only enabled service providers are displayed in the token.
        """
        # First we force the auth_data update via the set_auth() command
        # in the auth_provider
        self.auth_provider.set_auth()

        # Now we can retrieve the updated auth_data
        auth_data = self.auth_provider.get_auth()[1]
        try:
            return auth_data['service_providers']
        except KeyError:
            # no service providers in token
            return []



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from tempest.lib.common import rest_client

from keystone_tempest_plugin.services.identity import clients


class AuthClient(clients.Identity):

    def _get_scopes(self, url, token_id):
        resp, body = self.raw_request(
            url, 'GET', headers={'X-Auth-Token': token_id})
        self.expected_success(200, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def get_available_projects_scopes(self, keystone_v3_endpoint, token_id):
        """Get projects that are available to be scoped to based on a token."""
        url = '%s/auth/projects' % keystone_v3_endpoint
        return self._get_scopes(url, token_id)

    def get_available_domains_scopes(self, keystone_v3_endpoint, token_id):
        """Get domains that are available to be scoped to based on a token."""
        url = '%s/auth/domains' % keystone_v3_endpoint
        return self._get_scopes(url, token_id)



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacProjectsTests(rbac_base.IdentityV3RbacBaseTests,
                                  metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacProjectsTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.projects_client
        cls.users_client = cls.persona.users_v3_client
        cls.admin_client = cls.os_system_admin
        cls.admin_projects_client = cls.admin_client.projects_client

    @abc.abstractmethod
    def test_identity_create_project(self):
        """Test identity:create_project policy.

        This test must check:
          * whether the persona can create a project
          * whether the persona can create a project in their own domain
          * whether the persona can create a project in another domain
        """
        pass

    @abc.abstractmethod
    def test_identity_get_project(self):
        """Test identity:get_project policy.

        This test must check:
          * whether the persona can get a project
          * whether the persona can get a project in their own domain
          * whether the persona can get a project in another domain
          * whether the persona can get a project that does not exist
          * whether the persona can get their own project
        """
        pass

    @abc.abstractmethod
    def test_identity_list_projects(self):
        """Test identity:list_projects policy.

        This test must check:
          * whether the persona can list all projects
          * whether the persona can list all projects in their own domain
          * whether the persona can list all projects in another domain
        """
        pass

    @abc.abstractmethod
    def test_identity_list_user_projects(self):
        """Test identity:list_user_projects policy.

        This test must check:
          * whether the persona can list projects of a user
          * whether the persona can list projects of a user in their own domain
          * whether the persona can list projects of a user in another domain
          * whether the persona can list projects for themself
        """
        pass

    @abc.abstractmethod
    def test_identity_update_project(self):
        """Test identity:update_project policy.

        This test must check:
          * whether the persona can update a project
          * whether the persona can update a project in their own domain
          * whether the persona can update a project in another domain
          * whether the persona can update a project that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_project(self):
        """Test identity:delete_project policy.

        This test must check
          * whether the persona can delete a project
          * whether the persona can delete a project in their own domain
          * whether the persona can delete a project in another domain
          * whether the persona can delete a project that does not exist
        """
        pass


class SystemAdminTests(IdentityV3RbacProjectsTests, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_project(self):
        project_id = self.do_request(
            'create_project', expected_status=201, name=data_utils.rand_name()
        )['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)

    def test_identity_get_project(self):
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('show_project', project_id=project_id)
        # user gets a 404 for nonexistent project
        self.do_request('show_project', expected_status=exceptions.NotFound,
                        project_id=data_utils.rand_uuid_hex())

    def test_identity_list_projects(self):
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        resp = self.do_request('list_projects')
        self.assertIn(project_id, [p['id'] for p in resp['projects']])

    def test_identity_list_user_projects(self):
        user_id = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name())['user']['id']
        self.addCleanup(self.admin_client.users_v3_client.delete_user, user_id)
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        role_id = self.admin_client.roles_v3_client.create_role(
            name=data_utils.rand_name())['role']['id']
        self.addCleanup(self.admin_client.roles_v3_client.delete_role,
                        role_id)
        self.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id)
        # user can list projects for arbitrary user
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=user_id)
        self.assertIn(project_id, [p['id'] for p in resp['projects']])
        # user can list projects for self
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=self.persona.credentials.user_id)
        self.assertEqual(0, len([p['id'] for p in resp['projects']]))

    def test_identity_update_project(self):
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('update_project',
                        project_id=project_id,
                        description=data_utils.arbitrary_string())
        # user gets a 404 for nonexistent domain
        self.do_request('update_project', expected_status=exceptions.NotFound,
                        project_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_project(self):
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.do_request('delete_project', expected_status=204,
                        project_id=project_id)


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_project(self):
        self.do_request('create_project', expected_status=exceptions.Forbidden,
                        name=data_utils.rand_name())

    def test_identity_update_project(self):
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('update_project', expected_status=exceptions.Forbidden,
                        project_id=project_id,
                        description=data_utils.arbitrary_string())
        # user gets a 403 for nonexistent domain
        self.do_request('update_project', expected_status=exceptions.Forbidden,
                        project_id=data_utils.rand_uuid_hex())

    def test_identity_delete_project(self):
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('delete_project', expected_status=exceptions.Forbidden,
                        project_id=project_id)


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(IdentityV3RbacProjectsTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def setUp(self):
        super(DomainAdminTests, self).setUp()
        self.own_domain = self.persona.credentials.domain_id
        self.other_domain = self.admin_client.domains_client.create_domain(
            name=data_utils.rand_name())['domain']['id']
        self.addCleanup(self.admin_client.domains_client.delete_domain,
                        self.other_domain)
        self.addCleanup(self.admin_client.domains_client.update_domain,
                        domain_id=self.other_domain, enabled=False)

    def test_identity_create_project(self):
        # user can create project in own domain
        project_id = self.do_request(
            'create_project', expected_status=201, name=data_utils.rand_name(),
            domain_id=self.own_domain
        )['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        # user can create project in other domain
        project_id = self.do_request(
            'create_project', expected_status=201, name=data_utils.rand_name(),
            domain_id=self.other_domain
        )['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)

    def test_identity_get_project(self):
        # user can get project in own domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('show_project', project_id=project_id)
        # user can get project in other domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('show_project', project_id=project_id)
        # user gets a 403 for nonexistent project
        self.do_request('show_project', expected_status=exceptions.NotFound,
                        project_id=data_utils.rand_uuid_hex())

    def test_identity_list_projects(self):
        # user can list projects but cannot see project in other domain
        own_project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project,
                        own_project_id)
        other_project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project,
                        other_project_id)
        resp = self.do_request('list_projects')
        self.assertIn(own_project_id, [d['id'] for d in resp['projects']])
        self.assertNotIn(other_project_id, [d['id'] for d in resp['projects']])

    def test_identity_list_user_projects(self):
        # user can list projects for user in own domain
        user_id = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['user']['id']
        self.addCleanup(self.admin_client.users_v3_client.delete_user, user_id)
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        role_id = self.admin_client.roles_v3_client.create_role(
            name=data_utils.rand_name())['role']['id']
        self.addCleanup(self.admin_client.roles_v3_client.delete_role,
                        role_id)
        self.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id)
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=user_id)
        self.assertIn(project_id, [p['id'] for p in resp['projects']])
        # user can list projects for user in other domain
        user_id = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['user']['id']
        self.addCleanup(self.admin_client.users_v3_client.delete_user, user_id)
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        role_id = self.admin_client.roles_v3_client.create_role(
            name=data_utils.rand_name())['role']['id']
        self.addCleanup(self.admin_client.roles_v3_client.delete_role,
                        role_id)
        self.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id)
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=user_id)
        self.assertIn(project_id, [p['id'] for p in resp['projects']])
        # user can list projects for self
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=self.persona.credentials.user_id)
        self.assertEqual(0, len([p['id'] for p in resp['projects']]))

    def test_identity_update_project(self):
        # user can update project in own domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('update_project',
                        project_id=project_id,
                        description=data_utils.arbitrary_string())
        # user can update project in other domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('update_project',
                        project_id=project_id,
                        description=data_utils.arbitrary_string())
        # user gets a 404 for nonexistent domain
        self.do_request('update_project', expected_status=exceptions.NotFound,
                        project_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_project(self):
        # user can delete project in own domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.do_request('delete_project', expected_status=204,
                        project_id=project_id)
        # user can delete project in other domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.do_request('delete_project', expected_status=204,
                        project_id=project_id)


class DomainManagerTests(DomainAdminTests):

    credentials = ['domain_manager', 'system_admin']

    def test_identity_create_project(self):
        # user can create project in own domain
        project_id = self.do_request(
            'create_project', expected_status=201,
            name=data_utils.rand_name(),
            domain_id=self.own_domain
        )['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        # user cannot create project in other domain
        self.do_request(
            'create_project', expected_status=exceptions.Forbidden,
            name=data_utils.rand_name(), domain_id=self.other_domain
        )

    def test_identity_get_project(self):
        # user can get project in own domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('show_project', project_id=project_id)
        # user cannot get project in other domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('show_project', expected_status=exceptions.Forbidden,
                        project_id=project_id)
        # user gets a 403 for nonexistent project
        self.do_request('show_project', expected_status=exceptions.Forbidden,
                        project_id=data_utils.rand_uuid_hex())

    def test_identity_list_user_projects(self):
        # user can list projects for user in own domain
        user_id = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['user']['id']
        self.addCleanup(self.admin_client.users_v3_client.delete_user, user_id)
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        role_id = self.admin_client.roles_v3_client.create_role(
            name=data_utils.rand_name())['role']['id']
        self.addCleanup(self.admin_client.roles_v3_client.delete_role,
                        role_id)
        self.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id)
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=user_id)
        self.assertIn(project_id, [p['id'] for p in resp['projects']])
        # user cannot list projects for user in other domain
        user_id = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['user']['id']
        self.addCleanup(self.admin_client.users_v3_client.delete_user, user_id)
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        role_id = self.admin_client.roles_v3_client.create_role(
            name=data_utils.rand_name())['role']['id']
        self.addCleanup(self.admin_client.roles_v3_client.delete_role,
                        role_id)
        self.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id)
        self.do_request('list_user_projects', client=self.users_client,
                        expected_status=exceptions.Forbidden,
                        user_id=user_id)
        # user can list projects for self
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=self.persona.credentials.user_id)
        self.assertEqual(0, len([p['id'] for p in resp['projects']]))

    def test_identity_update_project(self):
        # user can update project in own domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('update_project',
                        project_id=project_id,
                        description=data_utils.arbitrary_string())
        # user cannot update project in other domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('update_project',
                        expected_status=exceptions.Forbidden,
                        project_id=project_id,
                        description=data_utils.arbitrary_string())
        # user gets a 403 for nonexistent domain
        self.do_request('update_project', expected_status=exceptions.Forbidden,
                        project_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_project(self):
        # user can delete project in own domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.do_request('delete_project', expected_status=204,
                        project_id=project_id)
        # user cannot delete project in other domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('delete_project', expected_status=exceptions.Forbidden,
                        project_id=project_id)


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']

    def test_identity_create_project(self):
        # user cannot create project in own domain
        self.do_request(
            'create_project', expected_status=exceptions.Forbidden,
            name=data_utils.rand_name(),
            domain_id=self.own_domain
        )
        # user cannot create project in other domain
        self.do_request(
            'create_project', expected_status=exceptions.Forbidden,
            name=data_utils.rand_name(), domain_id=self.other_domain
        )

    def test_identity_get_project(self):
        # user can get project in own domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('show_project', project_id=project_id)
        # user cannot get project in other domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('show_project', expected_status=exceptions.Forbidden,
                        project_id=project_id)
        # user gets a 403 for nonexistent project
        self.do_request('show_project', expected_status=exceptions.Forbidden,
                        project_id=data_utils.rand_uuid_hex())

    def test_identity_list_user_projects(self):
        # user can list projects for user in own domain
        user_id = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['user']['id']
        self.addCleanup(self.admin_client.users_v3_client.delete_user, user_id)
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        role_id = self.admin_client.roles_v3_client.create_role(
            name=data_utils.rand_name())['role']['id']
        self.addCleanup(self.admin_client.roles_v3_client.delete_role,
                        role_id)
        self.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id)
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=user_id)
        self.assertIn(project_id, [p['id'] for p in resp['projects']])
        # user cannot list projects for user in other domain
        user_id = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['user']['id']
        self.addCleanup(self.admin_client.users_v3_client.delete_user, user_id)
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        role_id = self.admin_client.roles_v3_client.create_role(
            name=data_utils.rand_name())['role']['id']
        self.addCleanup(self.admin_client.roles_v3_client.delete_role,
                        role_id)
        self.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id)
        self.do_request('list_user_projects', client=self.users_client,
                        expected_status=exceptions.Forbidden,
                        user_id=user_id)
        # user can list projects for self
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=self.persona.credentials.user_id)
        self.assertEqual(0, len([p['id'] for p in resp['projects']]))

    def test_identity_update_project(self):
        # user cannot update project in own domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('update_project',
                        expected_status=exceptions.Forbidden,
                        project_id=project_id,
                        description=data_utils.arbitrary_string())
        # user cannot update project in other domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('update_project',
                        expected_status=exceptions.Forbidden,
                        project_id=project_id,
                        description=data_utils.arbitrary_string())
        # user gets a 403 for nonexistent domain
        self.do_request('update_project', expected_status=exceptions.Forbidden,
                        project_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_project(self):
        # user cannot delete project in own domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.do_request('delete_project', expected_status=exceptions.Forbidden,
                        project_id=project_id)
        # user cannot delete project in other domain
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.other_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('delete_project', expected_status=exceptions.Forbidden,
                        project_id=project_id)


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']

    def test_identity_list_user_projects(self):
        user_id = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name())['user']['id']
        self.addCleanup(self.admin_client.users_v3_client.delete_user, user_id)
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        role_id = self.admin_client.roles_v3_client.create_role(
            name=data_utils.rand_name())['role']['id']
        self.addCleanup(self.admin_client.roles_v3_client.delete_role,
                        role_id)
        self.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id)
        # user can list projects for arbitrary user
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=user_id)
        self.assertIn(project_id, [p['id'] for p in resp['projects']])
        # user can list projects for self
        # Project Admin is assigned to a tempest project so we cant re-use
        # the System Admin test.
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=self.persona.credentials.user_id)
        self.assertEqual(1, len([p['id'] for p in resp['projects']]))


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']

    def test_identity_get_project(self):
        # user cannot get arbitrary project
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name(),
            domain_id=self.own_domain)['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        self.do_request('show_project', expected_status=exceptions.Forbidden,
                        project_id=project_id)
        # user gets a 403 for nonexistent project
        self.do_request('show_project', expected_status=exceptions.Forbidden,
                        project_id=data_utils.rand_uuid_hex())
        # user can get own project
        self.do_request('show_project',
                        project_id=self.persona.credentials.project_id)

    def test_identity_list_projects(self):
        # user cannot list projects
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project,
                        project_id)
        self.do_request('list_projects', expected_status=exceptions.Forbidden)

    def test_identity_list_user_projects(self):
        # user can list projects for other user
        user_id = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name())['user']['id']
        self.addCleanup(self.admin_client.users_v3_client.delete_user, user_id)
        project_id = self.admin_projects_client.create_project(
            name=data_utils.rand_name())['project']['id']
        self.addCleanup(self.admin_projects_client.delete_project, project_id)
        role_id = self.admin_client.roles_v3_client.create_role(
            name=data_utils.rand_name())['role']['id']
        self.addCleanup(self.admin_client.roles_v3_client.delete_role,
                        role_id)
        self.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id)
        self.do_request('list_user_projects', client=self.users_client,
                        expected_status=exceptions.Forbidden,
                        user_id=user_id)
        # user can list projects for self
        resp = self.do_request('list_user_projects', client=self.users_client,
                               user_id=self.persona.credentials.user_id)
        self.assertIn(self.persona.credentials.project_id,
                      [p['id'] for p in resp['projects']])


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin import clients
from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacServiceProviderTests(rbac_base.IdentityV3RbacBaseTests,
                                         metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacServiceProviderTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        persona_mgr = clients.Manager(cls.persona.credentials)
        cls.client = persona_mgr.service_providers_client
        admin_client = cls.os_system_admin
        admin_mgr = clients.Manager(admin_client.credentials)
        cls.admin_sp_client = admin_mgr.service_providers_client

    @abc.abstractmethod
    def test_identity_create_service_provider(self):
        """Test identity:create_service_provider policy.

        This test must check:
          * whether the persona can create a service provider
        """
        pass

    @abc.abstractmethod
    def test_identity_get_service_provider(self):
        """Test identity:get_service_provider policy.

        This test must check:
          * whether the persona can get a service provider
          * whether the persona can get a service provider that does not
            exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_service_providers(self):
        """Test identity:list_service_providers policy.

        This test must check:
          * whether the persona can list all identity providers
        """
        pass

    @abc.abstractmethod
    def test_identity_update_service_provider(self):
        """Test identity:update_service_provider policy.

        This test must check:
          * whether the persona can update a service provider
          * whether the persona can update a service provider that does not
            exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_service_provider(self):
        """Test identity:delete_service_provider policy.

        This test must check
          * whether the persona can delete a service provider
          * whether the persona can delete a service provider that does not
            exist
        """
        pass


class SystemAdminTests(IdentityV3RbacServiceProviderTests,
                       base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_service_provider(self):
        sp_id = self.do_request(
            'create_service_provider', expected_status=201,
            sp_id=data_utils.rand_name(),
            auth_url=data_utils.rand_url(),
            sp_url=data_utils.rand_url()
        )['service_provider']['id']
        self.addCleanup(self.admin_sp_client.delete_service_provider, sp_id)

    def test_identity_get_service_provider(self):
        sp_id = self.admin_sp_client.create_service_provider(
            sp_id=data_utils.rand_name(),
            auth_url=data_utils.rand_url(),
            sp_url=data_utils.rand_url())['service_provider']['id']
        self.addCleanup(self.admin_sp_client.delete_service_provider, sp_id)
        self.do_request('show_service_provider', sp_id=sp_id)
        # user gets a 404 for nonexistent sp
        self.do_request('show_service_provider',
                        expected_status=exceptions.NotFound,
                        sp_id=data_utils.rand_uuid_hex())

    def test_identity_list_service_providers(self):
        sp_id = self.admin_sp_client.create_service_provider(
            sp_id=data_utils.rand_name(),
            auth_url=data_utils.rand_url(),
            sp_url=data_utils.rand_url())['service_provider']['id']
        self.addCleanup(self.admin_sp_client.delete_service_provider, sp_id)
        resp = self.do_request('list_service_providers')
        self.assertIn(sp_id, [i['id'] for i in resp['service_providers']])

    def test_identity_update_service_provider(self):
        sp_id = self.admin_sp_client.create_service_provider(
            sp_id=data_utils.rand_name(),
            auth_url=data_utils.rand_url(),
            sp_url=data_utils.rand_url())['service_provider']['id']
        self.addCleanup(self.admin_sp_client.delete_service_provider, sp_id)
        self.do_request('update_service_provider',
                        sp_id=sp_id,
                        description=data_utils.arbitrary_string())
        # user gets a 404 for nonexistent sp
        self.do_request('update_service_provider',
                        expected_status=exceptions.NotFound,
                        sp_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_service_provider(self):
        sp_id = self.admin_sp_client.create_service_provider(
            sp_id=data_utils.rand_name(),
            auth_url=data_utils.rand_url(),
            sp_url=data_utils.rand_url())['service_provider']['id']
        self.do_request('delete_service_provider', expected_status=204,
                        sp_id=sp_id)
        # user gets a 404 for nonexistent sp
        self.do_request('delete_service_provider',
                        expected_status=exceptions.NotFound,
                        sp_id=sp_id)


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_service_provider(self):
        self.do_request('create_service_provider',
                        expected_status=exceptions.Forbidden,
                        sp_id=data_utils.rand_name(),
                        auth_url=data_utils.rand_url(),
                        sp_url=data_utils.rand_url())

    def test_identity_update_service_provider(self):
        sp_id = self.admin_sp_client.create_service_provider(
            sp_id=data_utils.rand_name(),
            auth_url=data_utils.rand_url(),
            sp_url=data_utils.rand_url())['service_provider']['id']
        self.addCleanup(self.admin_sp_client.delete_service_provider, sp_id)
        self.do_request('update_service_provider',
                        expected_status=exceptions.Forbidden,
                        sp_id=sp_id,
                        description=data_utils.arbitrary_string())
        # user gets a 403 for nonexistent sp
        self.do_request('update_service_provider',
                        expected_status=exceptions.Forbidden,
                        sp_id=data_utils.rand_uuid_hex(),
                        description=data_utils.arbitrary_string())

    def test_identity_delete_service_provider(self):
        sp_id = self.admin_sp_client.create_service_provider(
            sp_id=data_utils.rand_name(),
            auth_url=data_utils.rand_url(),
            sp_url=data_utils.rand_url())['service_provider']['id']
        self.addCleanup(self.admin_sp_client.delete_service_provider, sp_id)
        self.do_request('delete_service_provider',
                        expected_status=exceptions.Forbidden,
                        sp_id=sp_id)
        # user gets a 403 for nonexistent sp
        self.do_request('delete_service_provider',
                        expected_status=exceptions.Forbidden,
                        sp_id=sp_id)


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_service_provider(self):
        sp_id = self.admin_sp_client.create_service_provider(
            sp_id=data_utils.rand_name(),
            auth_url=data_utils.rand_url(),
            sp_url=data_utils.rand_url())['service_provider']['id']
        self.addCleanup(self.admin_sp_client.delete_service_provider, sp_id)
        self.do_request('show_service_provider',
                        expected_status=exceptions.Forbidden,
                        sp_id=sp_id)
        # user gets a 403 for nonexistent sp
        self.do_request('show_service_provider',
                        expected_status=exceptions.Forbidden,
                        sp_id=data_utils.rand_uuid_hex())

    def test_identity_list_service_providers(self):
        sp_id = self.admin_sp_client.create_service_provider(
            sp_id=data_utils.rand_name(),
            auth_url=data_utils.rand_url(),
            sp_url=data_utils.rand_url())['service_provider']['id']
        self.addCleanup(self.admin_sp_client.delete_service_provider, sp_id)
        self.do_request('list_service_providers',
                        expected_status=exceptions.Forbidden)


class DomainManagerTests(DomainAdminTests, base.BaseIdentityTest):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from lxml import etree
import requests


class Saml2Client(object):

    ECP_SP_EMPTY_REQUEST_HEADERS = {
        'Accept': 'text/html, application/vnd.paos+xml',
        'PAOS': ('ver="urn:liberty:paos:2003-08";"urn:oasis:names:tc:'
                 'SAML:2.0:profiles:SSO:ecp"')
    }

    ECP_SP_SAML2_REQUEST_HEADERS = {'Content-Type': 'application/vnd.paos+xml'}

    def __init__(self):
        self.reset_session()

    def reset_session(self):
        self.session = requests.Session()

    def _idp_auth_url(self, keystone_v3_endpoint, idp_id, protocol_id):
        subpath = 'OS-FEDERATION/identity_providers/%s/protocols/%s/auth' % (
            idp_id, protocol_id)
        return '%s/%s' % (keystone_v3_endpoint, subpath)

    def send_service_provider_request(self, keystone_v3_endpoint,
                                      idp_id, protocol_id):
        return self.session.get(
            self._idp_auth_url(keystone_v3_endpoint, idp_id, protocol_id),
            headers=self.ECP_SP_EMPTY_REQUEST_HEADERS
        )

    def prepare_sp_saml2_authn_response(self, saml2_idp_authn_response,
                                        relay_state):
        # Replace the header contents of the Identity Provider response with
        # the relay state initially sent by the Service Provider. The response
        # is a SOAP envelope with the following structure:
        #
        #   <S:Envelope
        #       <S:Header>
        #           ...
        #       </S:Header>
        #       <S:Body>
        #           ...
        #       </S:Body>
        #   </S:Envelope>
        saml2_idp_authn_response[0][0] = relay_state

    def send_identity_provider_authn_request(self, saml2_authn_request,
                                             idp_url, username, password):

        saml2_authn_request.remove(saml2_authn_request[0])
        return self.session.post(
            idp_url,
            headers={'Content-Type': 'text/xml'},
            data=etree.tostring(saml2_authn_request),
            auth=(username, password)
        )

    def send_service_provider_saml2_authn_response(
            self, saml2_idp_authn_response, idp_consumer_url):

        return self.session.post(
            idp_consumer_url,
            headers=self.ECP_SP_SAML2_REQUEST_HEADERS,
            data=etree.tostring(saml2_idp_authn_response),
            # Do not follow HTTP redirect
            allow_redirects=False
        )

    def send_service_provider_unscoped_token_request(self, sp_url):
        return self.session.get(
            sp_url,
            headers=self.ECP_SP_SAML2_REQUEST_HEADERS
        )
